<?php
require_once dirname(__FILE__).'/ISocialComponent.php';
require_once dirname(__FILE__).'/ISocialAdapter.php';
require_once dirname(__FILE__).'/AbstractSocialComponent.php';

class WordpressSocialAdapter implements ISocialComponent, ISocialAdapter
{
    const SOCIAL_KEY_GET_REQUEST = "social";
    
    private $_social;
    private $_engine;
    
    public function __construct(ISocialComponent $social, IFestiEngine $engine)
    {
        $this->_social = $social;
        $this->_engine = $engine;
    } // end __construct
    
    public function onLoginAction()
    {
        add_filter('the_content', array($this, 'onLoginContentPage'));
    } // end onLoginAction
    
    public function onSignupAction($data)
    {
        if (is_user_logged_in()) {
            throw new Exception("User already logged.");
        }

        if (!$this->_isValidSignupData($data)) {
            return false;
        }
        
        $userData = $data['userData'];

        $idUser = username_exists($userData['user_login']);
        if (is_null($idUser)) {
            $idUser = email_exists($userData['user_email']);
        }
    
        if (!$idUser) {
            return $this->_onSignupUser($data);
        } 
        
        return $this->_onSigninUser($data);
    } // end onSignupAction
    
    private function _onSigninUser($data)
    {
        $idUser = username_exists($data['userData']['user_login']);

        if (is_null($idUser)) {
            $idUser = $this->_getUserIdByEmail($data['userData']);
        }
        
        $singUpType = $this->_getSignUpType($idUser);

        if ($singUpType) {
            $this->_onSyncUser($data);
            $idUser = $this->_onSignonUser($data['userData']);
        } else {
            $email = $data['userData']['user_email'];
            
            if (!$this->isValidateUser($email, $data['token'])) {
                throw new Exception("Not found user.");
            }
            
            wp_set_auth_cookie($idUser, true); 
        }

        $metaData = array(
            'social_id' => $data['social_id'],
            'token'     => $data['token']
        );
        
        $key = 'social_'.$this->getName();
        
        add_user_meta($idUser, $key, $metaData, true);
        
        return true;
    } // end _onSigninUser
    
    private function _getUserIdByEmail($userData)
    {
        $idUser = email_exists($userData['user_email']);
        
        if (!$idUser) {
            throw new Exception("Not found user.");
        }
        
        return $idUser;
    } // end _getUserIdByEmail

    private function _getSignUpType($userId)
    {
        $key = 'sing_up_type';
        
        $value = get_user_meta($userId, $key);
        
        if (!$value) {
            return false;
        }
        
        return $value[0];
    } // end _getSignUpType
    
    private function _addSignUpType($userId)
    {
        $key = 'sing_up_type';
        $value = 'social';
        
        add_user_meta($userId, $key, $value, true);
    } // end _addSignUpType

    private function _onSignonUser($userData)
    {
        $userData['user_pass'] = md5(
            $userData['user_login'].$this->getOption('appSecret')
        );

        $creds = array(
            'user_login'    => $userData['user_login'],
            'user_password' => $userData['user_pass'],
            'remember'      => true
        );
        
        remove_all_actions('wp_login');
        $user = wp_signon($creds, false);
        
        if (is_wp_error($user)) {
            throw new Exception($user->get_error_message());
        }
        
        return $user->ID;
    } // end _onSyngonUser
    
    private function _onSyncUser(&$data)
    {
        $userData = &$data['userData'];
        
        $idUser = $this->_getUserIdByEmail($userData);
        
        $user = get_user_by('id', $idUser);
        $userData['user_login'] = $user->get('user_login');
        
        $values = array( 
            'ID' => $idUser, 
            'user_pass' => md5(
                $userData['user_login'].$this->getOption('appSecret')
            ) 
        );
            
        $user = wp_update_user($values);
        
        if (is_wp_error($user)) {
            throw new Exception($user->get_error_message());
        }
        
        return true;
    } // end _onSyncUser
    
    private function _onSignupUser($data)
    {
       $userData = $data['userData'];
        
        $userData['user_pass'] = md5(
            $userData['user_login'].$this->getOption('appSecret')
        );
        
        $idUser = wp_insert_user($userData);
        
        if (is_wp_error($idUser)) {
            throw new Exception($idUser->get_error_message());
        }
        
        $this->_addSignUpType($idUser);
        
        return $this->_onSigninUser($data);
    } // end _onSignupUser
    
    private function _isValidSignupData($data)
    {
        if (!array_key_exists('userData', $data)) {
            return false;
        }
        
        if (!array_key_exists('user_email', $data['userData'])) {
            return false;
        }
        
        if (!array_key_exists('user_login', $data['userData'])) {
            return false;
        }
        
        if (!array_key_exists('token', $data)) {
            return false;
        }
        
        return true;
    } // end _isValidSignupData
    
    
    public function onLoginContentPage($content)
    {
        $name = $this->getName();
        
        $args = array(
            self::SOCIAL_KEY_GET_REQUEST => array(
                'name'   => $name,
                'action' => 'signup'
            ),
        );

        $vars = array(
            'url' => $this->_engine->getUrl($args)
        );
        
        return $content.$this->fetch('login.phtml', $vars);
    } // end onLoginContentPage
    
    public function fetchLoginButton()
    {
        $name = $this->getName();
        
        $args = array(
            self::SOCIAL_KEY_GET_REQUEST => array(
                'name'   => $name,
                'action' => 'login'
            )
        );

        $vars = array(
            'text'  => $this->getOption('loginButtonText'),
            'url'   => $this->_engine->getUrl($args),
            'class' => $name,
            'logoUrl' => $this->_engine->getPluginImagesUrl($name.'/logo.png')
        );
        
        return $this->fetch('login_button.phtml', $vars);
    } // end fetchLoginButton
    
    public function fetchShareButton($type = false)
    {
        $name = $this->getName();
        
        $type = ($this->_isShareButtonTypeLong($type)) ? $type : '';

        $vars = array(
            'class' => $name,
            'url'   => $this->_getPageUrl(),
            'text'  => $this->getOption('sharingButtonText'),
            'logoUrl' => $this->_engine->getPluginImagesUrl($name.'/logo.png'),
            'type' => $type
        );
        
        return $this->fetch('share_button.phtml', $vars);
    } // end fetchLoginButton
    
    private function _isShareButtonTypeLong($type)
    {
        return $type = 'long';
    } // end _isShareButtonTypeLong
    
    private function _getPageUrl()
    {
        $serverUrl = $_SERVER["HTTP_HOST"].$_SERVER["REQUEST_URI"];
        $siteUrl = get_site_url();
        $url = $this->_getUrlProtocolType($siteUrl).$serverUrl;
        return $url;
    } // end _getPageUrl
    
    private function _getUrlProtocolType($url)
    {
        $type = substr($url, 0, 5);
        $type = str_replace(':', '', $type);
        
        return $type.'://';
    } // end _getUrlProtocolType
    
    public function fetchFooter()
    {
        return $this->fetch('footer.phtml');
    } // end fetchFooter
    
    protected function fetch($templateName, $vars = array())
    {
        $vars['instance'] = $this;
        $vars['provider'] = $this->_social;
        
        return $this->_engine->fetch($this->getName().'/'.$templateName, $vars);
    } // end fetch
    
    public function getName()
    {
        return $this->_social->getName();
    } // end getName
    
    public function getOption($key)
    {
        return $this->_social->getOption($key);
    } // end getOption
    
    public function isValidateUser($email, $token)
    {
        return $this->_social->isValidateUser($email, $token);
    } // end isValidateUser
    
    public function doRedirectToAuthPage($query)
    {
        return $this->_social->doRedirectToAuthPage($query);
    } // end getOption
}
